#ifndef MYX_LOG_SPDLOG_HPP_
#define MYX_LOG_SPDLOG_HPP_

#include <spdlog/sinks/null_sink.h>
#include <spdlog/sinks/syslog_sink.h>
#include <spdlog/sinks/basic_file_sink.h>
#include <spdlog/sinks/rotating_file_sink.h>
#include <spdlog/sinks/daily_file_sink.h>
// #include <spdlog/async.h>
// #include <spdlog/async_logger.h>

namespace myx {

namespace log {

class Logger
{
	std::string m_name;
	spdlog::logger m_logger;
	std::string m_baseFileName;
	std::size_t m_maxRotatingFileSize;
	std::size_t m_maxRotatingFilesCount;
	spdlog::level::level_enum m_outputLevel;
	spdlog::level::level_enum m_flushLevel;
	std::shared_ptr < spdlog::sinks::null_sink_st > m_nullSink;
	std::shared_ptr< spdlog::sinks::ansicolor_stdout_sink_st > m_stdoutSink;
	std::shared_ptr< spdlog::sinks::ansicolor_stderr_sink_st > m_stderrSink;
	std::shared_ptr< spdlog::sinks::syslog_sink_st > m_syslogSink;
	std::shared_ptr< spdlog::sinks::basic_file_sink_st > m_basicFileSink;
	std::shared_ptr< spdlog::sinks::rotating_file_sink_st > m_rotatingFileSink;
	std::shared_ptr< spdlog::sinks::daily_file_sink_st > m_dailyFileSink;

public:
	Logger( const std::string& name );

	std::shared_ptr< spdlog::logger > logger();
	std::string name() const;
	void setName( const std::string& name );
	std::string baseFileName() const;
	void setBaseFileName( const std::string& baseFileName );

	void enableNullSink();
	void disableNullSink();
	void enableStdoutSink();
	void disableStdoutSink();
	void enableStderrSink();
	void disableStderrSink();
	void enableSyslogSink();
	void disableSyslogSink();
	void enableBasicFileSink();
	void disableBasicFileSink();
	void enableRotatingFileSink();
	void disableRotatingFileSink();
	void enableDailyFileSink();
	void disableDailyFileSink();
	spdlog::level::level_enum outputLevel() const;
	void setOutputLevel( const spdlog::level::level_enum& outputLevel );
	spdlog::level::level_enum flushLevel() const;
	void setFlushLevel( const spdlog::level::level_enum& flushLevel );
}; // class Logger


/**
 * @brief default_logger
 * @return
 */
std::shared_ptr< spdlog::logger > default_logger();


template < class F, class ... Args >
void trace( F&& format, Args&& ... args ) noexcept
{
	default_logger()->trace( std::forward< F >( format ),
	                         std::forward< Args >( args )... );
}


template < class F, class ... Args >
void debug( F&& format, Args&& ... args ) noexcept
{
	default_logger()->debug( std::forward< F >( format ),
	                         std::forward< Args >( args )... );
}


template < class F, class ... Args >
void info( F&& format, Args&& ... args ) noexcept
{
	default_logger()->info( std::forward< F >( format ),
	                        std::forward< Args >( args )... );
}


template < class F, class ... Args >
void warn( F&& format, Args&& ... args ) noexcept
{
	default_logger()->warn( std::forward< F >( format ),
	                        std::forward< Args >( args )... );
}


template < class F, class ... Args >
void error( F&& format, Args&& ... args ) noexcept
{
	default_logger()->error( std::forward< F >( format ),
	                         std::forward< Args >( args )... );
}


template < class F, class ... Args >
void log( spdlog::level::level_enum lv, F&& format, Args&& ... args ) noexcept
{
	default_logger()->log( lv, std::forward< F >( format ),
	                       std::forward< Args >( args )... );
}

} // namespace log

} // namespace myx


#endif // ifndef MYX_LOG_SPDLOG_HPP_
